import nn_rsa

#生成公钥和私钥
#nn_rsa.NnRsa.create_key()

#传入文件名前缀，生成公钥和私钥存到文件里面
# nn_rsa.NnRsa.create_key(file_name_prefix='sky')


public_key = """
-----BEGIN RSA PUBLIC KEY-----
MIGJAoGBAL0964+AlfwZfe2u2fa9xyanYz+qzQHZrd0PWOefzWc0t7TZHTzsyexC
E44urJhd02Gwju1FeuS5TBSP27UyWOxbORQWbdHl3qjXZJ24L+1eA/CIUzOeJNvG
0syAWk6L887faMa9e0jkL8P3B0jsVkB7Y9d8q8vQolsHvvj5DS/PAgMBAAE=
-----END RSA PUBLIC KEY-----
"""
private_key = """
-----BEGIN RSA PRIVATE KEY-----
MIICYAIBAAKBgQC9PeuPgJX8GX3trtn2vccmp2M/qs0B2a3dD1jnn81nNLe02R08
7MnsQhOOLqyYXdNhsI7tRXrkuUwUj9u1MljsWzkUFm3R5d6o12SduC/tXgPwiFMz
niTbxtLMgFpOi/PO32jGvXtI5C/D9wdI7FZAe2PXfKvL0KJbB774+Q0vzwIDAQAB
AoGAUXjyDFMIv4NeEtEfZ7omnj6cXRG0tyI3Vx+/X+ENtmwc1xBOJSewezWrY6A2
kFZ8pec0cXRjR0t7NLMGSPXahxaQv6wU58rOeLhjCxhyN9bh+0jechbayjcOFq0A
CqeWVoLTMCJoKM4pqhTrMdZByQktLGwXRXhRq01Ja963W+kCRQDDSTwNi5NRdyny
bHPh2h+NsPAub0XlLkluiN6Vv0MGGg/nNyYBic2ZvMulBkACN7Gw/Jat54OfotWU
m23apdkrRhK9XQI9APgToSxk9G9ERr/Y+xVhnvXmY1dgOak3+hhugKCiho8JRrEw
j9uFel3VjGZ9Ri4kcOPWp5k8R4rl13WjGwJEP3LS5R+9LJHH+jDMccv5xoJ/dsbG
cdDZIOWtVAABWcEr+5aKbaOFPnb9v09Jiq7nt7ZJipUWyc4REKhzR7/s0EeDHS0C
PG53VzrMjagKUDLXrR7dRIqdF7ltyN9YZM6CTla5aI/DAq6eQEDxVAb6s7VjrV+Y
rIhYKnNqyzTHmkTw5wJFALDOgGTdgDQO7evWPY50gngczYP9MnEsBxHpBxdoPUqg
Thpj6P3Edw6iB1WzXxQqvho+bVTFaeaI0kvZk9UiVdGr+Mwm
-----END RSA PRIVATE KEY-----
"""

#加密/解密一起

# t1 = nn_rsa.NnRsa(private_key=private_key, public_key=public_key)
# miwen = t1.encrypt("森哥说多少个山东高速感受感受是的感受到公司给的谁告诉告诉告诉哥哥哥42山东高速")  # 加密
# mingwen = t1.decrypt(miwen)  # 解密

#只解密
# t2 = nn_rsa.NnRsa(private_key=private_key)
# miwen = "LrcQVAgjA7yD6+Kvh50ztWlLoefBVOBDrcpaL9uStsjAphqvPXncVHVktPkzuILkMnteutGFMNcwgRdy3ZifcC5rRtlbeJI94w7cSVgN1anU9O1Qx0du8A4r0TrvSQUTYhrz/L3+YhC0P6UmAkpRfM7WHOL8Okzs6rfPNhhAfJY="
# mingwen = t2.decrypt(miwen)  # 解密

#只加密
# t3 = nn_rsa.NnRsa(public_key=public_key)
# mingwen = "加密测试"
# miwen = t3.encrypt(mingwen)  # 加密






import rsa
import base64
import os
from rsa import common


class NnRsa:

    def __check_rsa_key(self, key: str, public_tag=True):
        func = rsa.PublicKey.load_pkcs1 if public_tag else rsa.PrivateKey.load_pkcs1
        try:
            obj = func(key.encode())
        except:
            raise Exception("公钥/私钥解析出错！")
        return obj

    def __init__(self, public_key: str = None, private_key: str = None):
        '''加密需要传公钥，解密需要传私钥'''
        self.public_key_obj = self.__check_rsa_key(public_key) if public_key else None
        self.private_key_obj = self.__check_rsa_key(private_key, False) if private_key else None

    def get_max_message_len(self, decrypt=False):
        # 计算加密/解密的最大长度
        # 消息加密/解密的时候消息如果超过最大长度，解密/加密的时候需要一块一块解密/加密

        reserve_size = 0 if decrypt else 11  # 如果是加密的话，有一个预留的长度，是11，如果是解密，就没有预留长度
        rsa_key = self.private_key_obj.n if decrypt else self.public_key_obj.n
        # private_key_obj.n 是私钥的最大消息长度，public_key_obj.n是公钥最大的消息长度
        return common.byte_size(rsa_key) - reserve_size  # 最大长度为消息最大长度 - 预留长度

    def encrypt(self, message: str):
        '''公钥加密，传入需要加密的字符串'''
        if self.public_key_obj == None:
            raise Exception("加密时，公钥不能为空！")
        print("加密前字符串:{}".format(message))
        max_message_len = self.get_max_message_len()  # 获取最大长度
        result_message = bytes()  # rsa加密完的数据是bytes类型的
        message = message.encode()
        while message:  # 消息不为空的时候循环
            _message = message[:max_message_len]  # 从开头取到最大的消息长度
            message = message[max_message_len:]  # 从取完的下标往后取
            result_message += rsa.encrypt(_message, self.public_key_obj)  # 加密一块，累加到reuslt里面
        b64_msg = base64.b64encode(result_message).decode()  # 加密后的是bytes类型的，不是一个字符串，转成base64的
        print("加密后字符串:{}".format(b64_msg))
        return b64_msg

    def decrypt(self, message: str):
        '''私钥解密，传入加密后的字符串'''
        if self.private_key_obj == None:
            raise Exception("解密时，私钥不能为空！")
        print("解密前的字符串:{}".format(message))
        max_message_len = self.get_max_message_len(True)  # 获取最大长度
        decrypt_message = bytes()
        try:
            message = base64.b64decode(message)  # 加密之后，字符串是base64过的，base64解密一下
            while message:
                _message = message[:max_message_len]  # 从开头取到最大的消息长度
                message = message[max_message_len:]  # 从取完的下标往后取
                decrypt_message += rsa.decrypt(_message, self.private_key_obj)
                # 加密后的是bytes类型的，不是一个字符串，转成字符串，累加
        except:
            print("解密出错，请检查密文是否有误")
            return False
        print("解密成功，解密后的字符串：{}".format(decrypt_message.decode()))
        return decrypt_message.decode()

    @staticmethod
    def create_key(size=1024, file_name_prefix: str = None):
        if file_name_prefix:
            public_key_file_name = file_name_prefix + "_public_key.pem"
            private_key_file_name = file_name_prefix + "_private_key.pem"
            if os.path.exists(public_key_file_name) or os.path.exists(private_key_file_name):
                ok = input("公钥/私钥文件已经存在，是否需要覆盖，输入1覆盖，输入其他取消生成:").strip()
                if ok != "1":
                    return

        public_key_obj, private_key_obj = rsa.newkeys(size)
        public_key = public_key_obj.save_pkcs1()
        private_key = private_key_obj.save_pkcs1()

        if file_name_prefix == None:
            print("public_key:\n", public_key.decode())
            print("private_key:\n", private_key.decode())
            return

        with open(public_key_file_name, 'wb') as f1, open(private_key_file_name, 'wb') as f2:
            f1.write(public_key)
            f2.write(private_key)
        print("public_key_file:", public_key_file_name)
        print("private_key:", private_key_file_name)




# -*- coding: utf-8 -*-
import codecs

import os

try:
    from setuptools import setup

except:

    from distutils.core import setup

"""

打包的用的setup必须引入，

"""

NAME = "nn_rsa"

"""

名字，一般放你包的名字即可

"""

PACKAGES = ["nn_rsa", ]

"""

包含的包，可以多个，这是一个列表

"""

DESCRIPTION = "rsa加密"

"""

关于这个包的描述

"""

LONG_DESCRIPTION = '这是一个基于rsa模块模块封装的加密模块'

"""

参见read方法说明

"""

KEYWORDS = "rsa nn_rsa""""

关于当前包的一些关键字，方便PyPI进行分类。

"""

AUTHOR = "HanyangNiu"

AUTHOR_EMAIL = "niuhanyang@163.com"

URL = "http://www.nnzhp.cn/"
"""

你这个包的项目地址，如果有，给一个吧，没有你直接填写在PyPI你这个包的地址也是可以的

"""

VERSION = "1.0.2"

"""

当前包的版本，这个按你自己需要的版本控制方式来

"""

LICENSE = "MIT"

"""

授权方式，我喜欢的是MIT的方式，你可以换成其他方式

"""

setup(

    name=
    NAME,

    version=
    VERSION,

    description=
    DESCRIPTION,

    long_description=
    LONG_DESCRIPTION,

    classifiers=
    [

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python',

        'Intended Audience :: Developers',

        'Operating System :: OS Independent',

    ],

    keywords=
    KEYWORDS,

    author=
    AUTHOR,

    author_email=
    AUTHOR_EMAIL,

    url=
    URL,

    license=
    LICENSE,

    packages=
    PACKAGES,

    include_package_data=True,
    install_requires=[
        "rsa >= 4.7.2", ],

    zip_safe=True,

)

## 把上面的变量填入了一个setup()中即可。
